//=========================================================
// src/Interrupts.c: generated by Hardware Configurator
//
// This file will be regenerated when saving a document.
// leave the sections inside the "$[...]" comment tags alone
// or they will be overwritten!
//=========================================================

// USER INCLUDES
#include <SI_EFM8UB1_Register_Enums.h>		// SFR declarations
#include "LCDText_Defs.h"

/*The LCD expects data in least-significant-bit-first format, whereas the EFM8 can only perform
most-significant-bit-first SPI transfers. This look-up table allows us to use a normal 8-bit variable
for the address: the address is used as the index to this array, and the corresponding array value
is a bit-reversed address that can be sent out the SPI*/
code unsigned char LCDLineAddr_LUT[] =
{
  0x80, 0x40, 0xC0, 0x20, 0xA0, 0x60, 0xE0, 0x10, 0x90, 0x50, 0xD0, 0x30, 0xB0, 0x70, 0xF0,
  0x08, 0x88, 0x48, 0xC8, 0x28, 0xA8, 0x68, 0xE8, 0x18, 0x98, 0x58, 0xD8, 0x38, 0xB8, 0x78, 0xF8,
  0x04, 0x84, 0x44, 0xC4, 0x24, 0xA4, 0x64, 0xE4, 0x14, 0x94, 0x54, 0xD4, 0x34, 0xB4, 0x74, 0xF4,
  0x0C, 0x8C, 0x4C, 0xCC, 0x2C, 0xAC, 0x6C, 0xEC, 0x1C, 0x9C, 0x5C, 0xDC, 0x3C, 0xBC, 0x7C, 0xFC,
  0x02, 0x82, 0x42, 0xC2, 0x22, 0xA2, 0x62, 0xE2, 0x12, 0x92, 0x52, 0xD2, 0x32, 0xB2, 0x72, 0xF2,
  0x0A, 0x8A, 0x4A, 0xCA, 0x2A, 0xAA, 0x6A, 0xEA, 0x1A, 0x9A, 0x5A, 0xDA, 0x3A, 0xBA, 0x7A, 0xFA,
  0x06, 0x86, 0x46, 0xC6, 0x26, 0xA6, 0x66, 0xE6, 0x16, 0x96, 0x56, 0xD6, 0x36, 0xB6, 0x76, 0xF6,
  0x0E, 0x8E, 0x4E, 0xCE, 0x2E, 0xAE, 0x6E, 0xEE, 0x1E, 0x9E, 0x5E, 0xDE, 0x3E, 0xBE, 0x7E, 0xFE,
  0x01
};

//global variables
unsigned char SPITxCount;
unsigned char LCDLineAddress;
unsigned char LCDLineDataIndex;
bit UPDATE_LCD = FALSE;



//-----------------------------------------------------------------------------
// SPI0_ISR
//-----------------------------------------------------------------------------
//
// SPI0 ISR Content goes here. Remember to clear flag bits:
// SPI0CN0::MODF (Mode Fault Flag)
// SPI0CN0::RXOVRN (Receive Overrun Flag)
// SPI0CN0::SPIF (SPI# Interrupt Flag)
// SPI0CN0::WCOL (Write Collision Flag)
// SPI0FCN1::RFRQ (Receive FIFO Request)
// SPI0FCN1::TFRQ (Transmit FIFO Request)
//
//-----------------------------------------------------------------------------
SI_INTERRUPT (SPI0_ISR, SPI0_IRQn)
{
	SFRPAGE = SPI0_PAGE;
	SPI0CN0_SPIF = 0;															//clear the interrupt flag

	/*This is the state machine that handles the SPI transfers, implemented with a switch statement.
	 The value of LCDTxState determines which instructions are executed. One switch statement is used
	 for both all clear transfers and line data transfers. A flowchart that clarifies the operation of this state machine
	 is provided in the article */
	switch(LCDTxState)
	{
		case ALL_CLEAR_SENT:
			SFRPAGE = SPI0_PAGE;
			SPI0DAT = 0x00;
			LCDTxState = FIRST_ALLCLEAR_DUMMY_SENT;
			break;

		case FIRST_ALLCLEAR_DUMMY_SENT:
			SFRPAGE = SPI0_PAGE;
			SPI0DAT = 0x00;
			LCDTxState = SECOND_ALLCLEAR_DUMMY_SENT;
			break;

		case SECOND_ALLCLEAR_DUMMY_SENT:
			SFRPAGE = TIMER4_PAGE; TMR4L = 0; while(TMR4L < 10);			//this provides the required delay between the last clock cycle and chip select going low (thSCS in the LCD data sheet)
			SPI_CS = LOW;																							//set chip select LOW to complete the transfer
			SFRPAGE = TIMER4_PAGE; TMR4L = 0; while(TMR4L < 30);			//this ensures adequate delay between chip select going low then high again (twSCSL in the LCD data sheet)
			LCDTxState = IDLE;
			break;

		case DATA_UPDATE_MODE_SENT:
			LCDLineAddress = 0;
			SFRPAGE = SPI0_PAGE;
			SPI0DAT = LCDLineAddr_LUT[LCDLineAddress];
			LCDTxState = FIRST_LINE_ADDR_SENT;
			LCDLineDataIndex = 0;
			break;

		case FIRST_LINE_ADDR_SENT:
			SFRPAGE = SPI0_PAGE;
			SPI0DAT = LCDDisplayData[LCDLineAddress][LCDLineDataIndex];
			LCDLineDataIndex++;
			if(LCDLineDataIndex == NUM_LINE_DATA_BYTES)
				{LCDTxState = LINE_DATA_SENT;
				LCDLineAddress++;
				LCDLineDataIndex = 0;}
			break;

		case LINE_DATA_SENT:
			SFRPAGE = SPI0_PAGE;
			SPI0DAT = 0x00;
			LCDTxState = PRE_ADDR_DUMMY_SENT;
			break;

		case PRE_ADDR_DUMMY_SENT:
			SFRPAGE = SPI0_PAGE;
			SPI0DAT = LCDLineAddr_LUT[LCDLineAddress];
			LCDTxState = NEW_ADDR_SENT;
			break;

		case NEW_ADDR_SENT:
			SFRPAGE = SPI0_PAGE;
			SPI0DAT = LCDDisplayData[LCDLineAddress][LCDLineDataIndex];
			LCDLineDataIndex++;
			if(LCDLineDataIndex == NUM_LINE_DATA_BYTES)
			{
				LCDTxState = LINE_DATA_SENT;
				LCDLineAddress++;
				LCDLineDataIndex = 0;
				if(LCDLineAddress == NUM_LINES)
					LCDTxState = LAST_LINE_DATA_SENT;
			}
			break;

		case LAST_LINE_DATA_SENT:
			SFRPAGE = SPI0_PAGE;
			SPI0DAT = 0x00;
			LCDTxState = FIRST_POST_DATA_DUMMY_SENT;
			break;

		case FIRST_POST_DATA_DUMMY_SENT:
			SFRPAGE = SPI0_PAGE;
			SPI0DAT = 0x00;
			LCDTxState = SECOND_POST_DATA_DUMMY_SENT;
			break;

		case SECOND_POST_DATA_DUMMY_SENT:
			SFRPAGE = TIMER4_PAGE; TMR4L = 0; while(TMR4L < 10); 		//this provides the required delay between the last clock cycle and chip select going low (thSCS in the LCD data sheet)
			SPI_CS = LOW;																							//set chip select LOW to complete the transfer
			SFRPAGE = TIMER4_PAGE; TMR4L = 0; while(TMR4L < 30);			//this ensures adequate delay between chip select going low then high again (twSCSL in the LCD data sheet)
			LCDTxState = IDLE;
			break;
	}

}


//-----------------------------------------------------------------------------
// TIMER2_ISR
//-----------------------------------------------------------------------------
//
// TIMER2 ISR Content goes here. Remember to clear flag bits:
// TMR2CN0::TF2H (Timer # High Byte Overflow Flag)
// TMR2CN0::TF2L (Timer # Low Byte Overflow Flag)
//
//-----------------------------------------------------------------------------
SI_INTERRUPT (TIMER2_ISR, TIMER2_IRQn)
{
	SFRPAGE = TIMER2_PAGE;
	TMR2CN0_TF2H = 0;

	//this creates a blinking LED that gives quick confirmation that the interrupt is firing
	if (RGBLED_B == HIGH)
		{RGBLED_R = LOW; RGBLED_G = HIGH; RGBLED_B = LOW;}
	else
		{RGBLED_R = HIGH; RGBLED_G = HIGH; RGBLED_B = HIGH;}

	if(LCDTxState == IDLE)
		UPDATE_LCD = TRUE;			//this flag tells the InsertCharacter() function to initiate an LCD update

}

